package redirect_test

import (
	"encoding/json"
	"errors"
	"net/http/httptest"
	"testing"
	"url-shortener/internal/http-server/handlers/url/redirect"
	"url-shortener/internal/http-server/handlers/url/redirect/mocks"
	"url-shortener/internal/lib/api"
	resp "url-shortener/internal/lib/api/response"
	"url-shortener/internal/lib/logger/handlers/slogdiscard"
	"url-shortener/internal/storage"

	"github.com/go-chi/chi/v5"
	"github.com/stretchr/testify/assert"
	"github.com/stretchr/testify/require"
)

func TestRedirectHandler(t *testing.T) {
	// test table
	cases := []struct {
		name      string
		alias     string
		url       string
		respError string
		mockError error
	}{
		{
			name:  "Success",
			alias: "test_alias",
			url:   "https://www.google.com/",
		},
		{
			name:      "Not Found",
			alias:     "test_alias",
			mockError: storage.ErrURLNotFound,
			respError: "not found",
		},
		{
			name:      "Database error",
			alias:     "test_alias",
			mockError: errors.New("ERROR"),
			respError: "internal error",
		},
	}

	for _, tc := range cases {
		t.Run(tc.name, func(t *testing.T) {
			// create mocks
			urlGetterMock := mocks.NewURLProvider(t)

			if tc.respError == "" || tc.mockError != nil {
				urlGetterMock.On("GetURL", tc.alias).
					Return(tc.url, tc.mockError).Once()
			}
			// create handler
			r := chi.NewRouter()
			r.Get("/{alias}", redirect.New(slogdiscard.NewDiscardLogger(), urlGetterMock))

			ts := httptest.NewServer(r)
			defer ts.Close()
			var redirectedToURL string
			// Если ошибки быть не должно - проверить перенаправление
			// Если ошибка быть должна - проверить ошибку
			if tc.respError == "" {
				// check redirection
				var err error
				redirectedToURL, err = api.GetRedirect(ts.URL + "/" + tc.alias)
				require.NoError(t, err)
			} else {
				// create http recorder
				rr := httptest.NewRecorder()
				// создать http запрос
				req := httptest.NewRequest("GET", ts.URL+"/"+tc.alias, nil)
				// обработать http запрос
				r.ServeHTTP(rr, req)
				// получить тело ответа
				body := rr.Body.String()

				var resp resp.Response
				require.NoError(t, json.Unmarshal([]byte(body), &resp))
				require.Equal(t, tc.respError, resp.Error)
			}
			// Check the final URL after redirection.
			assert.Equal(t, tc.url, redirectedToURL)
		})
	}
}
